/* Blinker
   Copyright 2011 Stefan Schuermans <stefan@blinkenarea.org>
   Copyleft GNU public license - http://www.gnu.org/copyleft/gpl.html
   a blinkenarea.org project */

#ifndef BLINKER_FLEXIPIX_H
#define BLINKER_FLEXIPIX_H

#include <string>

#include <BlinkenLib/BlinkenFrame.h>

#include "Directory.h"
#include "File.h"
#include "InStreamFile.h"
#include "Mgrs.h"
#include "Module.h"
#include "Size.h"
#include "StreamRecv.h"
#include "Time.h"
#include "TimeCallee.h"

namespace Blinker {

/// FlexiPix output module
class FlexiPix: public Module, public StreamRecv, public TimeCallee
{
public:
  /**
   * @brief constructor
   * @param[in] mgrs managers
   * @param[in] dirBase base directory
   */
  FlexiPix(Mgrs &mgrs, const Directory &dirBase);

  /// virtual destructor
  virtual ~FlexiPix();

private:
  /// copy constructor disabled
  FlexiPix(const FlexiPix &that);

  /// assignment operator disabled
  const FlexiPix & operator=(const FlexiPix &that);

public:
  /// check for update of configuration
  virtual void updateConfig();

  /**
   * @brief set current frame
   * @param[in] stream stream name
   * @param[in] pFrame current frame (NULL for none)
   */
  virtual void setFrame(const std::string &stream, stBlinkenFrame *pFrame);

  /// callback when requested time reached
  virtual void timeCall();

protected:
  /// (re-)create FlexiPix display
  void createDisplay();

  /// destroy FlexiPix display
  void destroyDisplay();

  /**
   * @brief display frame on FlexiPix
   * @param[in] pFrame frame to display (or NULL)
   */
  void displayFrame(stBlinkenFrame *pFrame);

  /// (re-)send frame to FlexiPix
  void sendFrame();

protected:
  InStreamFile         m_fileInStream; ///< input stream name file
  File                 m_fileConfig;   ///< FlexiPix config file
  struct flp_display_s *m_pDisplay;    ///< FlexiPix display
  Size                 m_size;         ///< size of FlexiPix display
}; // class FlexiPix

} // namespace Blinker

#endif // #ifndef BLINKER_FLEXIPIX_H

