/* Blinker
   Copyright 2011 Stefan Schuermans <stefan@blinkenarea.org>
   Copyleft GNU public license - http://www.gnu.org/copyleft/gpl.html
   a blinkenarea.org project */

#ifndef BLINKER_OUTSTREAMFILE_H
#define BLINKER_OUTSTREAMFILE_H

#include <BlinkenLib/BlinkenFrame.h>

#include "File.h"
#include "StreamFile.h"
#include "StreamMgr.h"

namespace Blinker {

/// setting file containting a name of an output stream
class OutStreamFile: public StreamFile
{
public:
  /**
   * @brief constructor from path
   * @param[in] path path to file
   * @param[in] streamMgr stream manager
   */
  OutStreamFile(const std::string &path, StreamMgr &streamMgr);

  /**
   * @brief constructor from basic file
   * @param[in] file basic file object
   * @param[in] streamMgr stream manager
   */
  OutStreamFile(const File &file, StreamMgr &streamMgr);

  /// destructor
  ~OutStreamFile();

  /**
   * @brief assignment operator
   * @param[in] file basic file object
   */
  const OutStreamFile & operator=(const File &file);

private:
  /// copy constructor disabled
  OutStreamFile(const OutStreamFile &that);

  /// assignment operator disabled
  const OutStreamFile & operator=(const OutStreamFile &that);

public:
  /// update, i.e. (re-)read file and reference new stream
  void update();

  /**
   * @brief set current frame
   * @param[in] pFrame current frame (NULL for none)
   */
  void setFrame(stBlinkenFrame *pFrame);
}; // class OutStreamFile

} // namespace Blinker

#endif // #ifndef BLINKER_OUTSTREAMFILE_H

