/* Blinker
   Copyright 2011 Stefan Schuermans <stefan@blinkenarea.org>
   Copyleft GNU public license - http://www.gnu.org/copyleft/gpl.html
   a blinkenarea.org project */

#ifndef BLINKER_RESIZER_H
#define BLINKER_RESIZER_H

#include <string>

#include <BlinkenLib/BlinkenFrame.h>

#include "Directory.h"
#include "File.h"
#include "Format.h"
#include "FormatFile.h"
#include "InStreamFile.h"
#include "Mgrs.h"
#include "Module.h"
#include "OutStreamFile.h"
#include "StreamRecv.h"

namespace Blinker {

/// a stream resizer
class Resizer: public Module, public StreamRecv
{
public:
  /**
   * @brief constructor
   * @param[in] mgrs managers
   * @param[in] dirBase base directory
   */
  Resizer(Mgrs &mgrs, const Directory &dirBase);

  /// virtual destructor
  virtual ~Resizer();

private:
  /// copy constructor disabled
  Resizer(const Resizer &that);

  /// assignment operator disabled
  const Resizer & operator=(const Resizer &that);

public:
  /// check for update of configuration
  virtual void updateConfig();

  /**
   * @brief set current frame
   * @param[in] stream stream name
   * @param[in] pFrame current frame (NULL for none)
   */
  virtual void setFrame(const std::string &stream, stBlinkenFrame *pFrame);

protected:
  /// (re-)get format to resize to
  void getFormat();

  /// send current frame to output stream
  void sendFrame();

  /**
   * @brief process frame
   * @param[in] pFrame frame to process (NULL for none)
   */
  void procFrame(stBlinkenFrame *pFrame);

protected:
  InStreamFile  m_fileInStream;  ///< input stream name file
  FormatFile    m_fileFormat;    ///< format file
  OutStreamFile m_fileOutStream; ///< output stream name file
}; // class Resizer

} // namespace Blinker

#endif // #ifndef BLINKER_RESIZER_H

