/* Blinker
   Copyright 2011 Stefan Schuermans <stefan@blinkenarea.org>
   Copyleft GNU public license - http://www.gnu.org/copyleft/gpl.html
   a blinkenarea.org project */

#ifndef BLINKER_SCALER_H
#define BLINKER_SCALER_H

#include <string>

#include <BlinkenLib/BlinkenFrame.h>

#include "Directory.h"
#include "File.h"
#include "InStreamFile.h"
#include "Mgrs.h"
#include "Module.h"
#include "OutStreamFile.h"
#include "Size.h"
#include "SizeFile.h"
#include "StreamRecv.h"

namespace Blinker {

/// a stream scaler
class Scaler: public Module, public StreamRecv
{
public:
  /**
   * @brief constructor
   * @param[in] mgrs managers
   * @param[in] dirBase base directory
   */
  Scaler(Mgrs &mgrs, const Directory &dirBase);

  /// virtual destructor
  virtual ~Scaler();

private:
  /// copy constructor disabled
  Scaler(const Scaler &that);

  /// assignment operator disabled
  const Scaler & operator=(const Scaler &that);

public:
  /// check for update of configuration
  virtual void updateConfig();

  /**
   * @brief set current frame
   * @param[in] stream stream name
   * @param[in] pFrame current frame (NULL for none)
   */
  virtual void setFrame(const std::string &stream, stBlinkenFrame *pFrame);

protected:
  /// (re-)get size to scale to
  void getSize();

  /// send current frame to output stream
  void sendFrame();

  /**
   * @brief process frame
   * @param[in] pFrame frame to process (NULL for none)
   */
  void procFrame(stBlinkenFrame *pFrame);

protected:
  InStreamFile  m_fileInStream;  ///< input stream name file
  SizeFile      m_fileSize;      ///< size file
  OutStreamFile m_fileOutStream; ///< output stream name file
}; // class Scaler

} // namespace Blinker

#endif // #ifndef BLINKER_SCALER_H

