/* Blinker
   Copyright 2011 Stefan Schuermans <stefan@blinkenarea.org>
   Copyleft GNU public license - http://www.gnu.org/copyleft/gpl.html
   a blinkenarea.org project */

#include <sstream>
#include <string>

#include "Size.h"
#include "StringParser.h"

namespace Blinker {

/// constructor
Size::Size():
  m_width(1),
  m_height(1)
{
}

/**
 * @brief parse from string format
 * @param[in] str string format
 * @return if parsing was successful
 */
bool Size::fromStr(const std::string &str)
{
  StringParser parser(str);
  unsigned int width, height;

  if (!parser.uintMin(1, width) ||
      !parser.fixChr('x') ||
      !parser.uintMin(1, height) ||
      !parser.isDone())
    return false;
  m_width = width;
  m_height = height;
  return true;
}

/**
 * @brief convert to string format
 * @return string format
 */
std::string Size::toStr() const
{
  std::stringstream strm;
  strm << m_width << "x" << m_height;
  return strm.str();
}

} // namespace Blinker

