/* Blinker
   Copyright 2011 Stefan Schuermans <stefan@blinkenarea.org>
   Copyleft GNU public license - http://www.gnu.org/copyleft/gpl.html
   a blinkenarea.org project */

#include <iostream>
#include <string>

#include "Canvas.h"
#include "Directory.h"
#include "FlexiPix.h"
#include "Mgrs.h"
#include "ModuleMgr.h"
#include "ModuleMgr_impl.h"
#include "Output.h"
#include "Player.h"
#include "Printer.h"
#include "Priority.h"
#include "Resizer.h"
#include "Scaler.h"
#include "Udp4Receiver.h"
#include "Udp4Sender.h"

using namespace Blinker;

void run(const std::string &dirConfig)
{
  Directory cfg(dirConfig);

  Mgrs mgrs;

  ModuleMgr<Canvas>       canvases(mgrs,      cfg.getSubdir("canvases"));
  ModuleMgr<FlexiPix>     flexipixes(mgrs,    cfg.getSubdir("flexipixes"));
  ModuleMgr<Output>       outputs(mgrs,       cfg.getSubdir("outputs"));
  ModuleMgr<Player>       players(mgrs,       cfg.getSubdir("players"));
  ModuleMgr<Printer>      printers(mgrs,      cfg.getSubdir("printers"));
  ModuleMgr<Priority>     priorities(mgrs,    cfg.getSubdir("priorities"));
  ModuleMgr<Resizer>      resizers(mgrs,      cfg.getSubdir("resizers"));
  ModuleMgr<Scaler>       scalers(mgrs,       cfg.getSubdir("scalers"));
  ModuleMgr<Udp4Receiver> udp4Receivers(mgrs, cfg.getSubdir("udp4receivers"));
  ModuleMgr<Udp4Sender>   udp4Senders(mgrs,   cfg.getSubdir("udp4senders"));

  mgrs.m_callMgr.run();
}

int main(int argc, const char *argv[])
{
  const char * dirConfig;

  if (argc < 2) {
    std::cerr << "Blinker" << std::endl
              << "configuration:"BLINKER_CONFIG << std::endl
              << "usage: " << argv[0] << " config-directory" << std::endl;
    return 2;
  }
  dirConfig = argv[1];

  run(dirConfig);

  return 0;
}

